# -*- coding: utf-8 -*-
from django import template
register = template.Library()
from greeking.placekittens import get_url
from greeking.lorem_ipsum import words, paragraphs
from greeking.jabberwocky import get_grafs, get_html as get_jabberwocky_html
from greeking.pangrams import get_pangram, get_html as get_pangram_html
from greeking import quotables


class PlaceKittenNode(template.Node):
    def __init__(self, width=200, height=200, color=True):
        self.width = width
        self.height = height
        self.color = color
    
    def render(self, context):
        return '<img src="%s"/>' % get_url(self.width, self.height, color=self.color)


def placekitten(parser, token):
    """
    Creates an image of a random kitten at the provided width and height.
    
    Usage format:
        
        {% placekitten [width] [height] [gray] %}
        
    Example usage:
        
        Color image at 250 wide and 400 high
        {% placekitten 250 400 %}
        
        Grayscale image 100 wide and 100 high
        {% placekitten 100 100 gray %}
        
    """
    bits = list(token.split_contents())
    if len(bits) > 4 or len(bits) < 3:
        raise template.TemplateSyntaxError("Incorrect format")
    tagname, width, height = bits[:3]
    if len(bits) == 4:
        if bits[3] == 'gray':
            color = False
        else:
            raise template.TemplateSyntaxError("Incorrect format")
    else:
        color = True
    return PlaceKittenNode(width=width, height=height, color=color)
placekitten = register.tag(placekitten)


class CommentListNode(template.Node):
    def render(self, context):
        context['comment_list'] = quotables.get_comment_list()
        return ''


def greek_comment_list(parser, token):
    """ 
    Allows a template-level call of filler comments.

    Example usage:
    {% greek_comment_list as comment_list %}
    {% for comment in comment_list %}
        <div id="c{{ comment.id }}">
                <p>{{ comment.comment }}</p>
                <p>{{ comment.user_name }}</p>
                <p>{{ comment.submit_date|date:"F j, Y" }}</p>
                <p><a href="mailto:{{ comment.user_email }}">{{ comment.user_email }}</a></p>
                <p><a href="{{ comment.user_url }}">{{ comment.user_url }}</a></p>
        </div>
    {% endfor %}
    """
    return CommentListNode()
greek_comment_list = register.tag(greek_comment_list)


class LoremNode(template.Node):
    def __init__(self, count, method, common):
        self.count, self.method, self.common = count, method, common

    def render(self, context):
        try:
            count = int(self.count.resolve(context))
        except (ValueError, TypeError):
            count = 1
        if self.method == 'w':
            return words(count, common=self.common)
        else:
            paras = paragraphs(count, common=self.common)
        if self.method == 'p':
            paras = ['<p>%s</p>' % p for p in paras]
        return u'\n\n'.join(paras)


#@register.tag
def lorem(parser, token):
    """
    Creates random Latin text useful for providing test data in templates.

    Usage format::

        {% lorem [count] [method] [random] %}

    ``count`` is a number (or variable) containing the number of paragraphs or
    words to generate (default is 1).

    ``method`` is either ``w`` for words, ``p`` for HTML paragraphs, ``b`` for
    plain-text paragraph blocks (default is ``b``).

    ``random`` is the word ``random``, which if given, does not use the common
    paragraph (starting "Lorem ipsum dolor sit amet, consectetuer...").

    Examples:
        * ``{% lorem %}`` will output the common "lorem ipsum" paragraph
        * ``{% lorem 3 p %}`` will output the common "lorem ipsum" paragraph
          and two random paragraphs each wrapped in HTML ``<p>`` tags
        * ``{% lorem 2 w random %}`` will output two random latin words
    """
    bits = list(token.split_contents())
    tagname = bits[0]
    # Random bit
    common = bits[-1] != 'random'
    if not common:
        bits.pop()
    # Method bit
    if bits[-1] in ('w', 'p', 'b'):
        method = bits.pop()
    else:
        method = 'b'
    # Count bit
    if len(bits) > 1:
        count = bits.pop()
    else:
        count = '1'
    count = parser.compile_filter(count)
    if len(bits) != 1:
        raise template.TemplateSyntaxError("Incorrect format for %r tag" % tagname)
    return LoremNode(count, method, common)
lorem = register.tag(lorem)


class PangramNode(template.Node):
    def __init__(self, language):
        self.language = language

    def render(self, context):
        try:
            pangram = get_pangram(self.language)
        except KeyError:
            raise template.TemplateSyntaxError("Could not find a pangram for %r abbreviation" % self.language)
        return get_pangram_html(pangram)


#@register.tag
def pangram(parser, token):
    """
    Prints a pangram in the specified language.

    A pangram is a phrase that includes every letter of an alphabet.

    Default is English. For a full list of available languages, refer to pangrams.py

    Usage format::

        {% pangram [language] %}

    ``language`` is the two-letter abbreviation the desired language.

    Examples:
        * ``{% pangram %}`` will output the default English pangram.
        * ``{% pangram fr %}`` will output a French pangram.
    """
    bits = list(token.split_contents())
    tagname = bits[0]
    # Count bit
    if len(bits) > 1:
        language = bits[1]
    else:
        language = 'en'
    if len(bits) > 2:
        raise template.TemplateSyntaxError("Incorrect format for %r tag" % tagname)
    return PangramNode(language)
pangram = register.tag(pangram)


class JabberwockyNode(template.Node):
    def __init__(self, count):
        self.count = count

    def render(self, context):
        try:
            count = int(self.count)
        except (ValueError, TypeError):
            count = None
        return get_jabberwocky_html(get_grafs(count))


#@register.tag
def jabberwocky(parser, token):
    """
    Prints paragraphs from Lewis Caroll's poem Jabberwocky for greeking in templates.

    Usage format::

        {% jabberwocky [count] %}

    ``count`` is a number (or variable) containing the number of paragraphs or
    words to generate (default is 7, which prints the entire poem).

    Examples:
        * ``{% jabberwocky %}`` will output the common "lorem ipsum" paragraph
        * ``{% jabberwocky 3 %}`` will output three paragraphs from the poem
    """
    bits = list(token.split_contents())
    tagname = bits[0]
    # Count bit
    if len(bits) > 1:
        count = bits[1]
    else:
        count = None
    if len(bits) > 2:
        raise template.TemplateSyntaxError("Incorrect format for %r tag" % tagname)
    return JabberwockyNode(count)
jabberwocky = register.tag(jabberwocky)



# -*- coding: utf-8 -*-
"""
Utility functions for generating QuerySet object_lists that simulate data from django.contrib.comments
"""

import time
import datetime

def make_datetime(date_string):
    """Convert a string into a datetime object."""
    date_format = '%Y-%m-%d %H:%M'
    return datetime.datetime(*time.strptime(date_string, date_format)[0:5])

HIPHOP = {
    1: {'user_name': 'Chuck D', 'user_email': 'chuck@publicenemy.com', 'user_url': 'http://lyricwiki.org/Public_Enemy:Rebel_Without_A_Pause', 'comment': "No matter what the name - we're all the same. Pieces in one big chess game. Yeah - the voice of power is in the house - go take a shower boy.", 'submit_date': make_datetime('1988-06-26 00:01'),},
    2: {'user_name': 'Puff Daddy', 'user_email': 'diddy@badboy.com', 'user_url': 'http://www.seeklyrics.com/lyrics/Puff-Daddy/PE-2000.html', 'comment': "A bona fide playa, now who got the flavor. A non stop, rhythm rock, poetry sayer. I'm the life saver, the New York mayor. Before you try me, you better say your prayers.", 'submit_date': make_datetime('1999-12-31 23:59')},
    3: {'user_name': 'Rakim', 'user_email': 'rakim@ericbforpresident.com', 'user_url': 'http://www.lyricstime.com/eric-b-rakim-lyrics-of-fury-lyrics.html', 'comment': "The scene of a crime every night at the show. The fiend of a rhyme on the mic that you know. It’s only one capable, breaks-the unbreakable, melodies-unmakable, pattern-unecscapable.", 'submit_date': make_datetime('1998-07-25 00:01')},
    4: {'user_name': 'AZ', 'user_email': 'az@brooklyn.com', 'user_url': 'http://www.lyricsdepot.com/az/rather-unique.html', 'comment': "So it's gonna take more than your astrologist to knowledge this. A physiologist couldn't even figure out the exoticness. Raps demolishing certified the way I style it. My wordplay blaze with the rays of ultraviolence.", 'submit_date': make_datetime('1995-10-10 10:10')},
    5: {'user_name': 'Butterfly', 'user_email': 'butterfly@9thwonder.org', 'user_url': 'http://www.lyricstime.com/digable-planets-rebirth-of-slick-cool-like-dat-lyrics.html', 'comment': "How was the buzz entire hip hop era? Was fresh and fat since they started sayin’ Audi. Cause funks made fat from right beneath my hoodie. The puba of the styles like Miles and shit. Like sixties funky worms with waves and perms. Just sendin’ chunky rhythms right down ya block. We be to rap what key be to lock.", 'submit_date': make_datetime('1993-09-27 00:01')},
    6: {'user_name': 'Phife Dawg', 'user_email': 'diggy@canikick.it', 'user_url': 'http://www.lyricstime.com/a-tribe-called-quest-award-tour-lyrics.html', 'comment': "Comin’ with more hits than the Braves and the Yankees. Livin’ mad phat like an oversized Bam-bi. The wackest crews try to dis, it makes me laugh. When my track record’s longer than a DC-20 aircraft. So next time that you think you want somethin’ here. Make somethin’ differ, take that garbage to St. Elsewhere.", 'submit_date': make_datetime('1993-11-09 00:01')},
    7: {'user_name': 'Del', 'user_email': 'funkee@sunnymeadowz.com', 'user_url': 'http://www.lyricstime.com/del-the-funky-homosapien-sunny-meadowz-lyrics.html', 'comment': "I sit and write scriptures by the old wishin’ well. Collect all my notes and sail a boat back to Berkeley. Tries fill my vibe ’cause my style is rather earthly. Some say it's wack but I ain’t tryin’ to hear it. As long as what I do contains my soul and my spirit.", 'submit_date': make_datetime('1991-10-22 00:01')},
    8: {'user_name': 'Notorious B.I.G.', 'user_email': 'bee.eye.gee@brooklyn.com', 'user_url': 'http://www.azlyrics.com/lyrics/notoriousbig/ilovethedough.html', 'comment': "Now we buy homes in unfamiliar places. Tito smile everytime he see our faces.", 'submit_date': make_datetime('1997-03-25 00:16')},
    9: {'user_name': 'Jay-Z', 'user_email': 'jigga@brooklyn.com', 'user_url': 'http://www.lyricsmode.com/lyrics/s/scarface/guess_whos_back_feat_jay_z_beanie_sigel.html', 'comment': "Face it y'all, y'all n----- playin' basic ball. I'm on the block like I'm eight feet tall.", 'submit_date': make_datetime('2002-08-06 05:00')},
    10: {'user_name': 'Skoob', 'user_email': 'skoob@brooklyn.com', 'user_url': 'http://www.musicsonglyrics.com/D/dasefxlyrics/dasefxrealhiphoplyrics.htm', 'comment': 'We the ruffest dream team, reign Supreme like a Cutlass. Get duckets, the dough. Can\'t touch the flow.', 'submit_date': make_datetime('1995-08-26 12:01') },
}



class Quotable:
    """
    A dummy comment for use as template filler.
    
    Based on the models in django.contrib.comments.models
    """
    def __init__(self, comment_id, user_name, user_email, user_url, comment, submit_date):
        self.comment_id = comment_id
        self.user_name = user_name
        self.user_email = user_email
        self.user_url = user_url
        self.comment = comment
        self.submit_date = submit_date
        
    def print_comment(self):
        print "%s: %s" % (self.user_name, self.comment)


def get_comment_list():
    """
    Loop through a dictionary of dummy comments and pass them through the Quotable class.
    
    The idea is to simulate a Django queryset, which can then be passed into the template for greeking.
    """
    comment_list = []
    for k in HIPHOP.keys(): 
        q = Quotable(
                comment_id=k,
                user_name=HIPHOP[k]['user_name'],
                user_email=HIPHOP[k]['user_email'],
                user_url=HIPHOP[k]['user_url'],
                comment=HIPHOP[k]['comment'],
                submit_date=HIPHOP[k]['submit_date']
            )
        comment_list.append(q)
    return comment_list



# -*- coding: utf-8 -*-
"""
Utility for generating placeholder kitten images
"""
COLOR_URL = 'http://placekitten.com/%(width)s/%(height)s/'
GRAY_URL = 'http://placekitten.com/g/%(width)s/%(height)s/'

def get_url(width, height, color=True):
    """
    Craft the URL for a placekitten image.
    
    By default they are in color. To retrieve a grayscale image, set
    the color kwarg to False.
    """
    d = dict(width=width, height=height)
    if color:
        return COLOR_URL % d
    else:
        return GRAY_URL % d



from django.test import TestCase
from django.template import Template, Context
from django.utils.encoding import smart_unicode


class GreekingTemplateTagTests(TestCase):
    
    def render(self, t, **c):
        ctx = Context(c)
        out = Template(t).render(ctx)
        return ctx, out
    
    def testJabberwocky(self):
        """
        Tests the tag for pulling jabberwocky
        """
        from greeking.jabberwocky import get_grafs, get_html
        graph_range = range(1, 8)
        for graphs in graph_range:
            t = "{% load greeking_tags %}{% jabberwocky " + str(graphs) + " %}"
            ctx, out = self.render(t)
            match = get_html(get_grafs(graphs))
            self.assertEqual(out, match)
        
    def testPangrams(self):
        """
        Tests the tag for pulling pangrams
        """
        from greeking.pangrams import PANGRAMS, get_pangram, get_html
        languages = PANGRAMS.keys()
        for language in languages:
            t = "{% load greeking_tags %}{% pangram " + language + " %}"
            ctx, out = self.render(t)
            match = get_html(get_pangram(language))
            self.assertEqual(out, smart_unicode(match))
    
    def testGreekComments(self):
        """
        Tests the tag for pulling greek comments
        """
        from greeking import quotables
        t = "{% load greeking_tags %}{% greek_comment_list as comment_list %}"
        ctx, out = self.render(t)
        for comment in list(ctx['comment_list']):
            comment_dict = {
                'user_name': comment.user_name,
                'user_email': comment.user_email,
                'user_url': comment.user_url,
                'comment': comment.comment,
                'submit_date': comment.submit_date
            }
            match = quotables.HIPHOP[comment.comment_id]
            for k, v in comment_dict.items():
                self.assertEqual(v, match[k])
        
    def testPlaceKittens(self):
        """
        Tests the tag for pulling placekitten images.
        """
        t1 = "{% load greeking_tags %}{% placekitten 200 200 %}"
        ctx, out = self.render(t1)
        self.assertEqual(out, '<img src="http://placekitten.com/200/200/"/>')
        t2 = "{% load greeking_tags %}{% placekitten 200 200 gray %}"
        ctx, out = self.render(t2)
        self.assertEqual(out, '<img src="http://placekitten.com/g/200/200/"/>')




# -*- coding: utf-8 -*-
"""
Pull text from Lewis Carroll's poem Jabberwocky for template greeking.
"""

VERSE = """'Twas brillig, and the slithy toves
Did gyre and gimble in the wabe;
All mimsy were the borogoves,
And the mome raths outgrabe.

"Beware the Jabberwock, my son!
The jaws that bite, the claws that catch!
Beware the Jubjub bird, and shun
The frumious Bandersnatch!"

He took his vorpal sword in hand:
Long time the manxome foe he sought-
So rested he by the Tumtum tree,
And stood awhile in thought.

And as in uffish thought he stood,
The Jabberwock, with eyes of flame,
Came whiffling through the tulgey wood,
And burbled as it came!

One, two! One, two! and through and through
The vorpal blade went snicker-snack!
He left it dead, and with its head
He went galumphing back.

"And hast thou slain the Jabberwock?
Come to my arms, my beamish boy!
O frabjous day! Callooh! Callay!"
He chortled in his joy.

'Twas brillig, and the slithy toves
Did gyre and gimble in the wabe;
All mimsy were the borogoves,
And the mome raths outgrabe."""

def get_grafs(count=7):
    """
    Returns the specified number of paragraphs from "Jabberwocky."

    The default is seven, which will return the entire poem.
    """
    return VERSE.split("\n\n")[0:count]
    
def get_html(grafs):
    """
    Renders the grafs provided in HTML by wrapping them in <p> tags.
    
    Linebreaks are replaced with <br> tags.
    """
    html = ['<p>%s</p>' % p for p in grafs]
    html = [p.replace("\n", "<br>") for p in html]
    return u'\n\n'.join(html)